package com.jeeplus.modules.sg.overheadline.util;

import com.jeeplus.modules.sg.overheadline.entity.JkxlDetailFee;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DetailFeeUtil {

    /**
     * 处理节点基本信息，保证上级节点的‘直接工程费’、‘人工’等等费用与它的下级节点相应费用和相等
     * 处理方式是将上级节点与下级节点相应费用和的差额放入找到的第一个非空子节点。如果下级子节点金额全为空，则直接放到第一个子节点中
     *
     * @param detail 节点对象，这里要保证节点及所有子节点对象都不能为空，且都有wbs编码
     */
    public static void dealBaseInfo(JkxlDetailFee detail){
        List<JkxlDetailFee> subNodes = detail.getNodes();
        if(subNodes!=null && subNodes.size()>0){
            double total = 0.00;
            double zjgc = 0.00;
            double rg = 0.00;
            double cbr = 0.00;
            double fbr = 0.00;
            double jx = 0.00;
            for(int i=0;i<subNodes.size();i++){
                JkxlDetailFee node = subNodes.get(i);
                total = DoubleUtil.add(total,node.getOriginalTotalCost());
                zjgc = DoubleUtil.add(zjgc,node.getZjgcFee());
                rg = DoubleUtil.add(rg,node.getOriginalRgCost());
                cbr = DoubleUtil.add(cbr,node.getOriginalCbrCost());
                fbr = DoubleUtil.add(fbr,node.getOriginalFbrCost());
                jx = DoubleUtil.add(jx,node.getOriginalJxCost());
            }
            JkxlDetailFee firstNode = subNodes.get(0);
            firstNode.setOriginalTotalCost(DoubleUtil.add(firstNode.getOriginalTotalCost(),DoubleUtil.sub(detail.getOriginalTotalCost(),total)));
            firstNode.setZjgcFee(DoubleUtil.add(firstNode.getZjgcFee(),DoubleUtil.sub(detail.getZjgcFee(),zjgc)));
            firstNode.setOriginalRgCost(DoubleUtil.add(firstNode.getOriginalRgCost(),DoubleUtil.sub(detail.getOriginalRgCost(),rg)));
            firstNode.setOriginalCbrCost(DoubleUtil.add(firstNode.getOriginalCbrCost(),DoubleUtil.sub(detail.getOriginalCbrCost(),cbr)));
            firstNode.setOriginalFbrCost(DoubleUtil.add(firstNode.getOriginalFbrCost(),DoubleUtil.sub(detail.getOriginalFbrCost(),fbr)));
            firstNode.setOriginalJxCost(DoubleUtil.add(firstNode.getOriginalJxCost(),DoubleUtil.sub(detail.getOriginalJxCost(),jx)));
//            处理完后进行递归
            for(int i=0;i<subNodes.size();i++){
                dealBaseInfo(subNodes.get(i));
            }
        }
    }

    /**
     * 处理节点基本信息，保证上级节点的‘totalFee’与它的下级节点相应费用和相等
     * 处理方式是将上级节点与下级节点相应费用和的差额放入找到的第一个非空子节点。如果下级子节点金额全为空，则直接放到第一个子节点中
     *
     * @param detail 节点对象，这里要保证节点及所有子节点对象都不能为空，且都有wbs编码
     */
    public static void dealTotalFee(JkxlDetailFee detail){
        List<JkxlDetailFee> subNodes = detail.getNodes();
        if(subNodes!=null && subNodes.size()>0){
            double total = 0.00;
            for(int i=0;i<subNodes.size();i++){
                JkxlDetailFee node = subNodes.get(i);
                total = DoubleUtil.add(total,node.getTotalFee());
            }
            JkxlDetailFee firstNode = subNodes.get(0);
            firstNode.setTotalFee(DoubleUtil.add(firstNode.getTotalFee(),detail.getTotalFee(),-total));
//            处理完后进行递归
            for(int i=0;i<subNodes.size();i++){
                dealTotalFee(subNodes.get(i));
            }
        }
    }

    /**
     * 处理节点分本体信息，保证上级节点的‘措施费’、‘规费’等等费用与它的下级节点相应费用和相等
     * 处理方式是将上级节点与下级节点相应费用和的差额放入找到的第一个非空子节点。如果下级子节点金额全为空，则直接放到第一个子节点中
     *
     * @param detail 节点对象，这里要保证节点及所有子节点对象都不能为空，且都有wbs编码，都有值
     */
    public static void dealExtraInfo(JkxlDetailFee detail){
        List<JkxlDetailFee> subNodes = detail.getNodes();
        if(subNodes!=null && subNodes.size()>0){
            double csFee1 = 0.00;
            double csFee2 = 0.00;
            double csFee3 = 0.00;
            double qt = 0.00;
            double gf = 0.00;
            double cbrSb = 0.00;
            double tex = 0.00;
            double total = 0.00;
            for(int i=0;i<subNodes.size();i++){
                JkxlDetailFee node = subNodes.get(i);
                csFee1 = DoubleUtil.add(csFee1,node.getMeasuresFee1());
                csFee2 = DoubleUtil.add(csFee2,node.getMeasuresFee2());
                csFee3 = DoubleUtil.add(csFee3,node.getMeasuresFee3());
                qt = DoubleUtil.add(qt,node.getOtherCost());
                gf = DoubleUtil.add(gf,node.getFees());
                cbrSb = DoubleUtil.add(cbrSb,node.getCbrSb());
                tex = DoubleUtil.add(tex,node.getTex());
                total = DoubleUtil.add(total,node.getTotalFee());
            }
            JkxlDetailFee firstNode = subNodes.get(0);
            firstNode.setMeasuresFee1(DoubleUtil.add(firstNode.getMeasuresFee1(),detail.getMeasuresFee1(),-csFee1));
            firstNode.setMeasuresFee2(DoubleUtil.add(firstNode.getMeasuresFee2(),detail.getMeasuresFee2(),-csFee2));
            firstNode.setMeasuresFee3(DoubleUtil.add(firstNode.getMeasuresFee3(),detail.getMeasuresFee3(),-csFee3));
            firstNode.setOtherCost(DoubleUtil.add(firstNode.getOtherCost(),detail.getOtherCost(),-qt));
            firstNode.setFees(DoubleUtil.add(firstNode.getFees(),detail.getFees(),-gf));
            firstNode.setCbrSb(DoubleUtil.add(firstNode.getCbrSb(),detail.getCbrSb(),-cbrSb));
            firstNode.setTex(DoubleUtil.add(firstNode.getTex(),detail.getTex(),-tex));
            firstNode.setTotalFee(DoubleUtil.add(firstNode.getTotalFee(),detail.getTotalFee(),-total));
//            处理完后进行递归
            for(int i=0;i<subNodes.size();i++){
                dealExtraInfo(subNodes.get(i));
            }
        }
    }

    /**
     * 将节点对象fee，以及fee下所有层级子节点放入list中
     * @param fee
     * @param list
     */
    public static void addList(JkxlDetailFee fee,List<JkxlDetailFee> list){
        if(fee!=null){
            list.add(fee);
            List<JkxlDetailFee> subNodes = fee.getNodes();
            if(subNodes!=null&&subNodes.size()>0){
                for(int i=0;i<subNodes.size();i++){
                    JkxlDetailFee node = subNodes.get(i);
                    addList(node,list);
                }
            }
        }
    }

    /**
     * 将节点list整理到顶级节点下。这里要求所有list都在一个顶级list节点下
     * @param list
     * @return
     */
    public static JkxlDetailFee getTree(List<JkxlDetailFee> list){
        JkxlDetailFee topNode = null;
        Map<String,JkxlDetailFee> map = new HashMap<>();
        for(JkxlDetailFee fee:list){
            map.put(fee.getWbsCode(),fee);
        }
        for(JkxlDetailFee fee:list){
            String parentWbs = fee.getParentNode();
            JkxlDetailFee parentFee = map.get(parentWbs);
            if(parentFee==null){
                if(topNode==null){
                    topNode = fee;
                }else{
                    throw new RuntimeException("数据库中结算书节点结构异常！");
                }
            }else{
                List<JkxlDetailFee> subNodes = parentFee.getNodes();
                if(subNodes==null){
                    subNodes = new ArrayList<>();
                    parentFee.setNodes(subNodes);
                }
                subNodes.add(fee);
            }
        }
        return topNode;
    }

    /**
     * 逐级根据totalFee分摊金额至totalFee上。分摊后上下级和可能存在细微算术上导致的差额，可以用DealBaseInfo方法处理
     * @param top  分摊费用顶级节点
     * @param fee  分摊金额
     */
    public static void shareByTotal(JkxlDetailFee top,double fee){
        top.setTotalFee(DoubleUtil.add(top.getTotalFee(),fee));
        List<JkxlDetailFee> subNodes = top.getNodes();
        if(subNodes!=null&&subNodes.size()>0){
            for(JkxlDetailFee node:subNodes){
                double shareFee = DoubleUtil.round2(fee*node.getTotalFee()/top.getTotalFee());
                shareByTotal(node,shareFee);
            }
        }
    }

    /**
     * 逐级将金额平摊至所有节点上
     * @param top 分摊费用顶级节点
     * @param fee 分摊金额
     */
    public static void avgShareToTotal(JkxlDetailFee top,double fee){
        top.setTotalFee(DoubleUtil.add(top.getTotalFee(),fee));
        List<JkxlDetailFee> subNodes = top.getNodes();
        if(subNodes!=null&&subNodes.size()>0){
            int size = subNodes.size();
            double shareFee = DoubleUtil.round2(fee/size);
            for(JkxlDetailFee node:subNodes){
                avgShareToTotal(node,shareFee);
            }
        }
    }
}
